<?php

/**
 * Class WECR_Content_Filters
 *
 * This class deals with our content filters for restricting viewable content
 *
 * @since 0.0.1
 */
class WECR_Content_Filters{

	public function __construct(){

		add_filter( 'the_content', array( $this, 'restrict_content' ) );

		add_filter( 'wp_nav_menu_objects', array( $this, 'remove_page_from_menu' ), 1, 2 );

		add_filter( 'wp_list_pages_excludes', array( $this, 'restrict_these_pages' ) );

		// only run our code if bbPress exists
		if ( class_exists( 'bbPress' ) ){
			add_filter( 'bbp_current_user_can_access_create_topic_form', array( $this, 'topic_form_access' ), 10, 1 );
			add_filter( 'bbp_after_has_topics_parse_args', array( $this, 'topic_access' ), 10 );
			add_filter( 'bbp_get_reply_content', array( $this, 'hide_reply_content' ), 10, 2 );
			add_action( 'template_redirect', array( $this, 'apply_feedback_messages' ) );
		}

	} // __construct

	/**
	 * Override topic feedback message
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 * @hattip Pippins Plugins
	 *
	 * @param string        $translated_text    the original feedback text
	 * @param string        $text
	 * @param string        $domain             the textdomain to use
	 *
	 * @return string       $translated_text    the filtered feedback text
	 */
	public function topic_feedback_messages( $translated_text, $text, $domain ) {

		switch ( $translated_text ) {
			case 'You cannot reply to this topic.':
				$translated_text = apply_filters( 'wecr_restricted_content_message_cant_reply', $this->get_restricted_message() );
				break;
		}

		return wp_kses_post( $translated_text );

	} // topic_feedback_messages


	/**
	 * Override forum feedback message
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 * @hattip Pippins Plugins
	 *
	 * @param string    $translated_text        the original feedback text
	 * @param string    $text
	 * @param string    $domain                 the textdomain to use
	 *
	 * @return string   $translated_text        the filtered feedback text
	 */
	public function forum_feedback_messages( $translated_text, $text, $domain ) {

		switch ( $translated_text ) {
			case 'Oh bother! No topics were found here!':
				$translated_text = apply_filters( 'wecr_restricted_content_message_no_topics_found', $this->get_restricted_message() );
				break;
			case 'You cannot create new topics at this time.':
				$translated_text = apply_filters( 'wecr_restricted_content_message_cant_create_topics', $this->get_restricted_message() );
				break;
			case 'You must be logged in to create new topics.':
				$message = 'Topic creation is restricted to customers with a current purchase.';
				$translated_text = apply_filters( 'wecr_restricted_content_message_logged_in_create_topics', $this->get_restricted_message( $message ) );
				break;
		}

		return wp_kses_post( $translated_text );

	} // forum_feedback_messages


	/**
	 * Apply the filtered feedback messages to the forums
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 * @hattip Pippins Plugins
	 *
	 * @global  $user_ID
	 * @return  void
	 */
	public function apply_feedback_messages() {

		global $user_ID;

		if ( bbp_is_single_topic() ) {
			add_filter( 'gettext', array( $this, 'topic_feedback_messages' ), 20, 3 );
		} else if ( bbp_is_single_forum() && $this->forum_is_restricted( bbp_get_forum_id() ) ) {
			add_filter( 'gettext', array( $this, 'forum_feedback_messages' ), 20, 3 );
		}

	} // apply_feedback_messages

	/**
	 * Restricts the topics in a forum to users that have purchased a product
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param $query
	 * @return array
	 *
	 * @uses $this->user_has_admin_access()         Returns true if the current user can read posts
	 * @uses bbp_is_single_forum()                  Returns true if this is a single forum
	 * @uses $this->user_purchased_product()        Returns true if the user has purchased this product
	 * @uses bbp_get_forum_id()                     Returns the id of the forum
	 */
	public function topic_access( $query ){

		$is_restricted = true;

		if ( $this->user_has_admin_access() || ! bbp_is_single_forum() ) return $query;

		if ( $this->user_purchased_product( bbp_get_forum_id() ) ){
			$is_restricted = false;
		}

		if( $is_restricted ){
			// @todo this gives us some debug output that I need to find and kill
			return array(); // empty query
		} else {
			return $query;
		}

	} // topic_access

	/**
	 * Hides the reply content from the forums
	 *
	 * @since 1.2.2
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param string        $content        optional        Reply content
	 * @param int           $reply_id       optional        ID of the reply
	 * @uses $this->user_has_admin_access()                 Returns true if the user has admin access
	 * @uses $this->user_purchased_product()                Returns true if the user has purchased the product
	 * @uses $this->get_restricted_message()                Returns the restricted message if none given
	 * @return string
	 */
	public function hide_reply_content( $content, $reply_id ){

		if ( $this->user_has_admin_access() ) return $content;

		if ( $this->user_purchased_product( bbp_get_forum_id() ) ){
			return $content;
		}

		return $this->get_restricted_message( "This content is restricted to purchasers" );

	}

	/**
	 * Hides the topic form if you don't have access
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param bool  $value      Base value of bbp_current_user_can_access_create_topic_form
	 *
	 * @return bool             Our modified conditional value
	 *
	 * @uses $this->forum_is_restricted()           Returns true if the parent forum is restricted
	 * @uses $this->user_has_admin_access()         Returns true if the current user has the read cap assigned
	 * @uses $this->user_purchased_product()        Returns true if the user has purchased the product that we're restricted by
	 */
	public function topic_form_access( $value ){

		if ( ! $this->forum_is_restricted( get_the_ID() ) ) return true;
		if ( $this->user_has_admin_access() ) return true;
		if ( $this->user_purchased_product( get_the_ID()  ) ) return true;

		return false;

	} // topic_access

	/**
	 * Returns true if the parent forum of the topic is restricted.
	 *
	 * @param int   $post_id    required    post_id of the topic we're checking
	 *
	 * @return bool                         True if the forum is restricted
	 *
	 * @uses get_post_meta()                returns post meta given post_id and key
	 */
	private function forum_is_restricted( $post_id ){

		if ( get_post_type( $post_id ) === 'forum' ){
			$forum_id = absint( $post_id );
		} else {
			$forum_id = get_post_meta( $post_id, '_bbp_forum_id', true );
		}

		$restricted = get_post_meta( absint( $forum_id ), '_wecr_easy_restrict_content', true );

		if ( isset( $restricted ) && ! empty( $restricted ) ){
			return true;
		}

		return false;

	} // forum_is_restricted

	/**
	 * Restricts the pages from the wp_list_pages function which is the fallback for wp_nav_menu.
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param array     $exclude_array      required        The array of already excluded pages
	 *
	 * @return array    $exclude_array                      Our modified array of restricted pages
	 */
	public function restrict_these_pages( $exclude_array ){

		if ( $this->user_has_admin_access() ) return $exclude_array;

		$excluded = $this->get_restricted_pages();

		if ( empty( $excluded ) ) return $exclude_array;

		$excluded = $this->filter_pages_by_user_purchase( $excluded );

		return array_merge( $exclude_array, $excluded );

	} // restrict_these_pages

	/**
	 * Filters the excluded array and takes out pages that the user hasn't purchased the product for
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param array     $excluded   required    Original array of excluded page items
	 *
	 * @return array    $excluded_out           Our modified array of excluded items
	 *
	 * @uses $this->should_see_menu_item()      Returns true if the current_user should see the menu item
	 * @uses $this->should_page_be_restricetd() Returns true if the page should be removed from the menu
	 */
	private function filter_pages_by_user_purchase( $excluded ){

		$excluded_out = array();

		foreach( $excluded as $key => $value ){
			if ( $this->should_see_menu_item( $value ) || ! $this->should_page_be_restricted( $value ) ){
				continue;
			} else {
				$excluded_out[] = $value;
			}
		} // foreach

		return $excluded_out;

	} // filter_pages_by_user_purchase

	/**
	 * Returns true if the page should be removed from the menu
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int   $post_id    required    The post_id that we're checking
	 *
	 * @return bool                         True if the page should be removed  from the menu
	 *
	 * @uses get_post_meta()                Gets meta from post given post_id and key
	 */
	private function should_page_be_restricted( $post_id ){

		$restricted = get_post_meta( $post_id, '_wecr_remove_from_menu', true );

		if ( $restricted ){
			return true;
		}

		return false;

	} // should_page_be_restricted

	/**
	 * Returns an array of post_ids for restricted pages
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @return array        Array of post_ids for restricted pages
	 */
	private function get_restricted_pages(){

		$args = array(
			'post_type'         => 'page',
			'posts_per_page'    => -1,
			'meta_query'        => array(
				array(
					'key'       => '_wecr_easy_restrict_content',
					'compare'   => 'EXISTS'
				)
			)
		);

		$pages = get_posts( $args );

		return wp_list_pluck( $pages, 'ID' );

	} // get_restricted_pages

	/**
	 * Filters the menu items for restricted content
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param object|array  $items  required        Array of post object that are the individual menu item entries
	 * @param array         $args   optional        The nav menu args
	 *
	 * @return object|array                         Returns our filtered array of menu items
	 *
	 * @uses $this->should_see_menu_item()          Returns true if the current user should see the menu item
	 * @uses $this->should_page_be_restricted()     Returns true if the page should be removed from the menu
	 */
	public function remove_page_from_menu( $items, $args ){

		$items_in   = $items;
		$items_out  = array();

		foreach( $items_in as $i ){

			if ( $this->should_see_menu_item( $i->object_id ) || ! $this->should_page_be_restricted( $i->object_id ) ){
				$items_out[] = $i;
			} else {
				continue;
			}

		} // foreach

		return $items_out;

	} // remove_page_from_menu

	/**
	 * Returns true for admins or if the user has purchased the product
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param object    $post_id    required            post_id of the post we are checking
	 *
	 * @return bool                                     True if the user can see the menu item
	 *
	 * @uses $this->user_has_admin_access()             Returns true if the user has the cap to read content
	 * @uses $this->should_content_be_restricted()      Returns true if a product is set as a restriction for the given post_id
	 * @uses $this->user_purchased_product()            Returns true if the user has purchased the product restricted by the given post_id
	 */
	private function should_see_menu_item( $post_id ){

		if ( $this->user_has_admin_access() ){
			return true;
		}

		// if the user hasn't purchased the product return false
		if ( $this->should_content_be_restricted( $post_id ) ){
			if ( ! $this->user_purchased_product( $post_id ) ){
				return false;
			}
		}

		return true;

	} // should_see_menu

	/**
	 * Big daddy function that restricts content.
	 *
	 * Checks if we should restrict content then if we should it checks if the current user can see the page.
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param string        $content    required        Post/page content supplied by the_content filter
	 * @param int           $post_id    optional        The ID of the post we are checking for
	 * @uses $this->should_content_be_restricted()      Returns true if the content should be restricted
	 * @uses $this->user_has_admin_access()             Grants admin access to content (admin is never restricted)
	 * @uses $this->user_purchased_product()            Returns true if the user has purchased the product
	 * @uses $this->get_restricted_message()            Gets the message associated with a restricted post/page
	 * @return string       $content                    Our modified content or original content
	 */
	public function restrict_content( $content, $post_id = null ){

		if ( false === $this->should_content_be_restricted( $post_id ) ){
			return $content;
		}

		if ( $this->user_has_admin_access() || $this->user_purchased_product( $post_id ) ){
			return $content;
		}

		$content = $this->get_restricted_message();

		return $content;

	} // restrict_content

	/**
	 * A super early catch to see if a product has even been assigned to the content in question.
	 * If not then we can skip all our other checks.
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param   int     $post_id    optional        The post_id of the content we're checking
	 *
	 * @return bool                                 True if the content should be restricted
	 */
	private function should_content_be_restricted( $post_id = null ){

		$product_id = $this->get_restricted_product_id( $post_id );

		if ( isset( $product_id ) && ! empty( $product_id ) && is_array( $product_id ) ){
			return true;
		}

		return false;

	} // should_content_be_restricted

	/**
	 * Returns true if the current user has the capability to see restricted content
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @uses current_user_can()     Returns true if the current user has the given cap
	 */
	private function user_has_admin_access(){

		if ( current_user_can( 'wecr_read_content' ) ){
			return true;
		}

		return false;

	} // user_has_admin_access

	/**
	 * Returns true if the user has purchased the product that the page is restricted to
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param   int     $post_id    optional                        The post_id of the content we're checking
	 * @uses wp_get_current_user()                                  Returns WP_User object for current_user
	 * @uses $this->get_restricted_product_id()                     Returns id for restricted product
	 * @uses $this->is_grouped()                                    Returns true if this is a grouped product
	 * @uses $this->get_main_product()                              Returns product parent post_id
	 * @uses wc_customer_bougt_product()                            Returns true if customer has purchased product
	 * @uses WC_Subsriptions_Manage->user_has_subscription()        Returns true if the sure has a subscription
	 * @return bool     TRUE if the current user has purchased the product
	 */
	private function user_purchased_product( $post_id = null ){

		$current_user = wp_get_current_user();
		$product_ids  = $this->get_restricted_product_id( $post_id );

		foreach( $product_ids as $product_id ){

			// standard product detection
			if ( wc_customer_bought_product( $current_user->user_email, $current_user->ID, absint( $product_id ) ) ){
				return true;
			}

			// handling grouped product detection
			if ( $this->is_grouped( $product_id ) ){
				$parent_product_id = $this->get_main_product( $product_id );
				if ( wc_customer_bought_product( $current_user->user_email, $current_user->ID, absint( $parent_product_id ) ) ){
					return true;
				}
			}

			// subscription detection
			if ( class_exists( 'WC_Subscriptions_Manager' ) ){
				if ( WC_Subscriptions_Manager::user_has_subscription( $current_user->ID, $product_id, 'active' ) ) {
					return true;
				}
			}

		} // foreach

		return false;

	} // user_purchased_product

	/**
	 * Returns the restricted product ID
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis  McHale
	 * @access private
	 *
	 * @param   int     $post_id    optional        The post_id of the content we're checking
	 *
	 * @return int
	 */
	private function get_restricted_product_id( $post_id = null ){

		$post_id = isset( $post_id ) ? $post_id : get_the_ID();

		return get_post_meta( absint( $post_id ), '_wecr_easy_restrict_content', true );

	} // get_restricted_product_id

	/**
	 * Returns the content strings for content. May be different if content is restricted to a product purchase.
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param string    $content        required    The content we may change
	 *
	 * @return string   $content                    The modified content
	 *
	 * @uses $this->show_custom_message()                       Returns true if the custom message has content and the checkbox to show content is checked
	 * @uses $this->get_restricted_product_id()                 Returns the post_id of the product content is restricted by
	 * @uses get_post_meta()                                    Gets post meta given key and post_id
	 * @uses get_the_ID()                                       Gets current post_id inside the loop
	 * @uses get_permalink()                                    Gets permalink to post give post_id
	 * @uses get_the_title()                                    Returns the title of the given post_id
	 * @uses wp_kses_post()                                     Making our content safe for post output
	 * @uses WC_Subcriptions_Product->is_subscription()         Returns true if given product_id is a subscription
	 * @uses $this->get_subscription_message()                  Returns subscription message
	 * @uses $this->is_variation()                              Returns true if given product is a variation
	 */
	private function get_restricted_message( $content = null ){

		$product_ids     = $this->get_restricted_product_id();
		$custom_message = get_post_meta( get_the_ID(), '_wecr_custom_message', true );

		if ( $this->show_custom_message( $custom_message ) ){
			$content = $custom_message;
		} elseif ( isset( $content ) && ! is_null( $content ) ) {
			$content = $content;
		} else {

			$content = '';

			foreach( $product_ids as $product_id ){

				if ( class_exists( 'WC_Subcriptions_Product' ) ){
					$subscription_class = new WC_Subscriptions_Product();
					if (  $subscription_class->is_subscription( $product_id ) ){
						$content = apply_filters( 'wecr_subscription_message', $this->get_subscription_message( $product_id, $content ), $product_id );
					}
				}

				if ( $this->is_variation( $product_id ) && ! $this->is_grouped( $product_id ) ){

					$content = apply_filters( 'wecr_variable_product_message', $this->get_variable_product_message( $product_id, $content ), $product_id );

				} elseif ( $this->is_grouped( $product_id ) ){

					$content = apply_filters( 'wecr_grouped_product_message', $this->get_grouped_product_message( $product_id, $content ), $product_id );

				} else {

					if ( ! empty( $content ) ){
						$content .= __( ' or ', 'wecr' );
					} else {
						$content .= __( 'This content is restricted to users that have purchased ', 'wecr' );
					}
					$content .=  '<a href="'. get_permalink( absint( $product_id ) ) .'">'. get_the_title( absint( $product_id ) ) .'</a>';
					$content = apply_filters( 'wecr_simple_product_message', $content, $product_id );
				}

			} //foreach
		}

		$content .= '.';
		$content = apply_filters( 'wecr_restricted_content_message_global', $content, $product_ids );

		return wp_kses_post( $content );

	} // maybe_restrict_content

	/**
	 * Provides stock messaging around a grouped product restriction
	 *
	 * @since 1.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int       $product_id             required            ID of the product we are looking for
	 * @param string    $content                optional            Existing content if any
	 * @uses $this->get_main_product()                              returns post_id for the parent product
	 * @uses get_permalink()                                        Gets permalink given post_id
	 * @uses get_the_title()                                        Returns title given post_id
	 */
	private function get_grouped_product_message( $product_id, $content = '' ){

		$grouped_product_id = $product_id;
		$product_id = $this->get_main_product( $product_id );

		if ( ! empty( $content ) ){
			$content .= __( ' or ', 'wecr' );
		} else {
			$content .= __( 'This content is restricted to users that have purchased ', 'wecr' );
		}
		$content .=  '<a href="'. get_permalink( absint( $product_id ) ) .'">'. get_the_title( absint( $product_id ) ) .'</a>';

		$grouped_product = get_product( $grouped_product_id );
		if ( $grouped_product->is_visible() ){
			$content .= ' or <a href="'. get_permalink( absint( $grouped_product_id ) ) .'">'. get_the_title( absint( $grouped_product_id ) ) .'</a>';
		}

		return $content;
	}

	/**
	 * Returns a message for variable products
	 *
	 * @since 1.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int       $product_id         required        product id of the product we want a message for
	 * @param string    $content            optional        Existing content if any
	 * @uses absint()                                       No negative numbers
	 * @uses WC_Product_Variation()                         new product variation class
	 * @uses WP_Product->get_attributes()                   Gets product attributes
	 * @uses $this->sort_out_variable_name()                Figures out our name for a variable product
	 * @uses get_permalink()                                returns permalink given post_id
	 * @uses esc_attr()                                     Keeping out text safe
	 * @uses get_the_title()                                Returns the title of a post given id
	 * @return string       $content                        Our variable product message
	 */
	private function get_variable_product_message( $product_id, $content = '' ){

		$var_id = absint( $product_id );
		$vp = new WC_Product_Variation( $var_id );
		$product_id = $this->get_main_product( $product_id );

		$product_class = new WC_Product( $product_id );
		$attributes = $product_class->get_attributes();

		$variation_name = $this->sort_out_variable_name( $attributes, $vp, $product_id );

		if ( ! empty( $content ) ){
			$content .= __( ' or ', 'wecr' );
		} else {
			$content .= __( 'This content is restricted to users that have purchased ', 'wecr' );
		}
		$content .=  '<a href="'. get_permalink( absint( $product_id ) ) .'">'. get_the_title( absint( $product_id ) ) .'</a>. Choose the '. esc_attr( $variation_name ) .' option';

		return $content;

	} // get_variable_product_message

	/**
	 * Abstracts the message for subscription restricted products
	 *
	 * @since 1.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int       $product_id         required        product id of the product we want a message for
	 * @param string    $content            optional        Existing content if any
	 * @uses absint()                                       No negative numbers
	 * @uses WC_Product_Variation()                         new product variation class
	 * @uses WP_Product->get_attributes()                   Gets product attributes
	 * @uses $this->sort_out_variable_name()                Figures out our name for a variable product
	 * @uses get_permalink()                                returns permalink given post_id
	 * @uses esc_attr()                                     Keeping out text safe
	 * @uses get_the_title()                                Returns the title of a post given id
	 * @return string       $content                        Our variable product message
	 */
	private function get_subscription_message( $product_id, $content = '' ){

		if ( $this->is_variation( $product_id ) ) {

			$variable_product_id = absint( $product_id );
			$vp = new WC_Product_Variation( $variable_product_id );
			$product_id = $this->get_main_product( $product_id );

			$product_class = new WC_Product( $product_id );
			$attributes = $product_class->get_attributes();

			$variation_name = $this->sort_out_variable_name( $attributes, $vp, $product_id );

			if ( ! empty( $content ) ){
				$content .= __( ' or ', 'wecr' );
			} else {
				$content .= __( 'This content is restricted to subscribers of the ', 'wecr' );
			}
			$content .=  '<a href="'. get_permalink( absint( $product_id ) ) .'">'. get_the_title( absint( $product_id ) ) .'</a> '. esc_attr( $variation_name ) .' subscription';
			$content = apply_filters( 'wecr_variable_subscription_default_message', $content, $product_id );
		} else {

			if ( ! empty( $conent ) ){
				$content .= __( ' or ', 'wecr' );
			} else {
				$content .= __( 'This content is restricted to purchasers of the ', 'wecr' );
			}
			$content .=  '<a href="'. get_permalink( absint( $product_id ) ) .'">'. get_the_title( absint( $product_id ) ) .'</a> subscription';
			$content = apply_filters( 'wecr_simple_subscription_default_message', $content, $product_id );
		}

		return $content;

	} // get_subscription_message

	/**
	 * Gives us the name we want for variations
	 *
	 * @since 1.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param array         $attributes     required        Product variations
	 * @param class         $vp             required        WC_Product_Variation class
	 * @param int           $product_id     required        main product id
	 */
	private function sort_out_variable_name( $attributes, $vp, $product_id ){

			foreach( $attributes as $a ) {
				if ( $a['is_taxonomy'] ){

					$values = wc_get_product_terms( $product_id, $a['name'], array( 'fields' => 'names' ) );
					$explode = explode( ' ', $vp->get_formatted_name() );

				} else {

					$values = array_map( 'trim', explode( WC_DELIMITER, $a['value'] ) );
					$explode = explode( ' ', $vp->get_formatted_name() );

				}

				foreach( $explode as $e ){

					foreach( $values as $v ){
						if ( sanitize_title( $v ) === $e ){
							$variation_name = $v;
							break;
						}
					} // foreach values

				} // foreach explode

			} // foreach $attributes as $a

			return esc_attr( $variation_name );

	} // sort_out_variable_name

	/**
	 * Returns us the product_id for the parent product
	 *
	 * @since 1.2
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param int       $product_id         required        ID of the variation you want the parent product for
	 * @uses get_post()                                     Returns post_object given post_id
	 * @return int                                          ID of the main product
	 */
	private function get_main_product( $product_id ){

		$product = get_post( absint( $product_id ) );

		return (int) $product->post_parent;

	} // get_main_product

	/**
	 * Returns true if we are on a grouped product
	 *
	 * @since 1.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int       $product_id         required        ID of the product we are checking
	 * @uses get_product()                                  WC function that gives us a populated product
	 * @return bool                                         True if a grouped product
	 */
	private function is_grouped( $product_id ){

		$product = get_product( absint( $product_id ) );

		if ( is_object( $product->post ) && $product->post->post_parent ){
			$parent = get_product( $product->post->post_parent );
			if ( $parent->product_type === 'grouped' ){
				return true;
			}
		}

		return false;

	} // is_grouped

	/**
	 * Returns true if given product is a variation
	 *
	 * @since 1.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int   $product_id         required            ID of the product we are checking
	 * @uses get_product()                                  WC function that gets us all the product data
	 * @return bool                                         TRUE if it's a variation
	 */
	private function is_variation( $product_id ){

		$product = get_product( absint( $product_id ) );

		if ( is_object( $product->parent ) ){
			if ( $product->parent->product_type === 'variable' || $product->parent->product_type === 'variable-subscription' ){
				return true;
			}
		}

		return false;

	} // is_variation

	/**
	 * Returns true if we should show the custom message and if there is a custom message
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param string    $custom_message     optional        The custom message, if there is one
	 *
	 * @return bool     True if we should show the message
	 *
	 * @uses get_post_meta()                Returns post meta given post_id and key
	 * @uses get_the_ID()                   Gets the post_id inside the loop
	 */
	private function show_custom_message( $custom_message ){

		if ( ! isset( $custom_message ) || empty( $custom_message ) ) {
			return false;
		}

		$checked = get_post_meta( get_the_ID(), '_wecr_checked', true );
		if ( $checked !== 'on' ){
			return false;
		}

		return true;

	} // show_custom_message

} // WECR_Content_Filters

$GLOBALS['wecr_content_filters'] = new WECR_Content_Filters();